@@ -30,6 +30,8 @@
  *
  */
 
+/// <reference path="./text-annotations/custom-annotations/manager.js" />
+
 let settingsWindow = null;
 let aiModelsListWindow = null; 
 let aiModelEditWindow = null;
@@ -37,9 +39,11 @@ let customProvidersWindow = null;
 let summarizationWindow = null;
 let translateSettingsWindow = null;
 let helperWindow = null;
+let customAssistantWindow = null;
 
 let spellchecker = null;
 let grammar = null;
+let customAssistantManager = new CustomAssistantManager();
 
 window.getActionsInfo = function() {
 	let actions = [];
@@ -67,6 +71,9 @@ window.addSupportAgentMode = function(editorVersion) {
 		if (e.keyCode === 27 && textAnnotatorPopup) {
 			textAnnotatorPopup.close();
 		}
+		if (e.keyCode === 27 && customAnnotationPopup) {
+			customAnnotationPopup.close();
+		}
 
 		if (e.keyCode === 27 && helperWindow) {
 			helperWindow.close();
@@ -133,7 +140,7 @@ window.addSupportAgentMode = function(editorVersion) {
 				async function checkEndAction() {
 					if (!isSendedEndLongAction) {
 						await Asc.Editor.callMethod("EndAction", ["Block", "AI (" + requestEngine.modelUI.name + ")"]);
-						isSendedEndLongAction = true
+						isSendedEndLongAction = true;
 					}
 				}
 
@@ -171,25 +178,20 @@ window.addSupportAgentMode = function(editorVersion) {
 					});
 				}
 
+				let markdownStreamer = new MarkDownStreamer();
+
 				let isSupportStreaming = window.EditorHelper.isSupportStreaming;
-				let dataStream = "";
 				async function onStreamEvent(data, end) {
 					if (isSupportStreaming)
-						await Asc.Library.PasteText(data);
-					dataStream += data;
-					if (true === end && "" !== dataStream) {
-						await Asc.Library.PasteText(dataStream);
-						dataStream = "";
-					}
+						await markdownStreamer.onStreamChunk(data, end);
+					else if (end)
+						await Asc.Library.PasteText(data);					
 				}
 
 				let result = await requestEngine.chatRequest(copyMessages, false, async function(data) {
 					if (!data)
 						return;
 
-					if (isSupportStreaming)
-						await checkEndAction();
-					
 					let oldBuffer = buffer;
 					buffer += data;
 					if (checkBuffer && buffer.length >= bufferWait.length) {
@@ -199,18 +201,25 @@ window.addSupportAgentMode = function(editorVersion) {
 						}
 					}
 
+					if (isSupportStreaming && !checkBuffer)
+						await checkEndAction();
+
 					if (!checkBuffer)
 						await onStreamEvent(data);
 				});
 
+				if (!isSupportStreaming)
+					buffer = result;
+
 				if (checkBuffer && !buffer.startsWith(bufferWait)) {
 					checkBuffer = false;
-					await onStreamEvent(buffer, true);
 				}
 
-				if (!isSupportStreaming) {
-					await onStreamEvent("", true);
-				}
+				if (!isSupportStreaming && !checkBuffer)
+					await onStreamEvent(buffer, true);
+
+				if (isSupportStreaming)
+					markdownStreamer.onStreamEnd();
 
 				await checkEndAction();
 
@@ -266,11 +275,266 @@ window.addSupportAgentMode = function(editorVersion) {
 	});
 }
 
+async function initAssistants() {
+	let _this = window.Asc.plugin;
+	if (!_this.sendEvent)
+		return;
+
+	spellchecker = new SpellChecker(textAnnotatorPopup);
+	grammar = new GrammarChecker(textAnnotatorPopup);
+	JSON.parse(
+			localStorage.getItem("onlyoffice_ai_saved_assistants") || "[]"
+		).forEach(assistantData => {
+			customAssistantManager.createAssistant(assistantData);
+		});
+
+	_this.attachEditorEvent("onParagraphText", function(obj) {
+		if (!obj)
+			return;
+		
+		spellchecker.onChangeParagraph(obj["paragraphId"], obj["recalcId"], obj["text"], obj["annotations"]);
+		grammar.onChangeParagraph(obj["paragraphId"], obj["recalcId"], obj["text"], obj["annotations"]);
+
+		customAssistantManager.onChangeParagraph(obj["paragraphId"], obj["recalcId"], obj["text"], obj["annotations"]);
+	});
+
+	_this.attachEditorEvent("onFocusAnnotation", function(obj) {
+		if (!obj)
+			return;
+	});
+
+	_this.attachEditorEvent("onBlurAnnotation", function(obj) {
+		if (!obj)
+			return;
+
+		if ("spelling" === obj["name"])
+			spellchecker.onBlur();
+		else if ("grammar" === obj["name"]) 
+			grammar.onBlur();
+		else if ("customAssistant" === obj["name"].slice(0, 15)) {
+			const assistantId = obj["name"].slice(16);
+			customAssistantManager.onBlur(assistantId);
+		}	
+	});
+
+	_this.attachEditorEvent("onClickAnnotation", function(obj) {
+		if (!obj)
+			return;
+
+		if ("grammar" === obj["name"])
+			grammar.onClick(obj["paragraphId"], obj["ranges"]);
+		else if ("spelling" === obj["name"])
+			spellchecker.onClick(obj["paragraphId"], obj["ranges"]);
+		else if ("customAssistant" === obj["name"].slice(0, 15)) {
+			const assistantId = obj["name"].slice(16);
+			customAssistantManager.onClick(assistantId, obj["paragraphId"], obj["ranges"]);
+		}
+	});
+}
+
+async function initExternalProviders() {
+	let _this = window.Asc.plugin;
+	if (!_this.sendEvent)
+		return;
+	
+	function onLoadCustomExternalProviders(providers) {
+		for (let i = 0, len = providers.length; i < len; i++) {
+			let item = providers[i];
+			if (!item.name)
+				continue;
+
+			if (!item.content) {
+				let url = item.url || "[external]";
+				let key = item.key || "";
+				let addon = item.addon || "";
+
+				item.content = "\"use strict\";\n\
+class Provider extends AI.Provider {\n\
+constructor() {\n\
+	super(\"" + item.name + "\", \"" + url + "\", \"" + key + "\", \"" + addon + "\");\n\
+}\n\
+}";
+				AI.addExternalProvider(item.content);
+			}
+		}
+
+		if (0 < providers.length) {
+			AI.Storage.save();
+			AI.Storage.load();
+		}
+	}
+
+	_this.attachEditorEvent("ai_onCustomProviders", function(providers) {
+		onLoadCustomExternalProviders(providers);
+	});
+
+	_this.attachEditorEvent("ai_onCustomInit", function(obj) {
+		
+		if (obj.settingsLock !== undefined) {
+			let isSettingsRemoved = obj.settingsLock === "removed";
+			let isSettingsDisabled = obj.settingsLock === "disabled";
+
+			if (window.buttonSettings) {
+				if (window.buttonSettings.removed != isSettingsRemoved || 
+					window.buttonSettings.disabled != isSettingsDisabled) {
+					window.buttonSettings.removed = isSettingsRemoved;
+					window.buttonSettings.disabled = isSettingsDisabled;
+
+					Asc.Buttons.updateToolbarMenu(window.buttonMainToolbar.id, window.buttonMainToolbar.name, [window.buttonSettings]);
+				}
+			}
+		}
+
+		// mark model as external (not saved to localstorage)
+		if (obj.actions) {
+			for (let type in obj.actions) {
+				if (obj.actions[type] && obj.actions[type].model) {
+					if (!obj.actions[type].model.startsWith(AI.externalModelPrefix))
+						obj.actions[type].model = AI.externalModelPrefix + obj.actions[type].model;
+				}
+			}
+		}
+		if (obj.models) {
+			for (let i = 0, len = obj.models.length; i < len; i++) {
+				let model = obj.models[i];
+				if (model.id && !model.id.startsWith(AI.externalModelPrefix))
+					model.id = AI.externalModelPrefix + model.id;
+			}
+		}
+
+		// override the actions, if needed
+		if (obj.actions) {
+			let isActionsOverride = obj.actionsOverride === true;
+
+			for (let type in obj.actions) {
+				if (!AI.Actions[type])
+					continue;
+				
+				if (!AI.Actions[type].model || isActionsOverride)
+					AI.Actions[type].model = obj.actions[type].model;
+			}
+
+			AI.ActionsSave();
+		}
+
+		let isUpdate = false;
+		if (obj.providers) {
+			let customProviders = [];
+			for (let type in obj.providers) {
+				if (!obj.providers[type].name)
+					continue;
+				customProviders.push(obj.providers[type]);
+			}
+			if (customProviders.length > 0)
+				onLoadCustomExternalProviders(customProviders);
+			isUpdate = true;
+		}
+
+		if (obj.models) {
+			for (let i = 0, len = obj.models.length; i < len; i++) {
+				let model = obj.models[i];
+				let isFound = false;
+
+				for (let j = 0, jLen = AI.Models.length; j < jLen; j++) {
+					let testModel = AI.Models[j];
+
+					if (testModel.name === model.name && 
+						testModel.provider === model.provider &&
+						testModel.id === model.id) {
+						isFound = true;
+						AI.Models[j] = model;
+						break;
+					}
+				}
+
+				if (!isFound) {
+					if (!model.endpoints)
+						model.endpoints = [AI.Endpoints.Types.v1.Chat_Completions];
+					AI.Models.push(model);
+				}
+			}
+			isUpdate = true;
+		}
+
+		if (isUpdate)
+			AI.Storage.save();
+		
+	});
+
+	_this.attachEditorEvent("ai_onCallTool", async function(toolCall) {
+		let funcName = toolCall.name;
+		let funcArgs = toolCall.arguments;
+		let argsObj = typeof funcArgs === 'string' ? JSON.parse(funcArgs) : funcArgs;
+
+		await Asc.Editor.callMethod("StartAction", ["GroupActions"]);
+
+		let toolResult = {};
+		try {
+			toolResult = await window.EditorHelper.names2funcs[funcName].call(argsObj);
+			if (!toolResult)
+				toolResult = {};
+			toolResult.message = "System function '" + funcName + "' executed successfully";
+		} catch (e) {
+			let errorMsg = "Error calling function: " + funcName;
+			console.error(errorMsg);
+			toolResult = {
+				error: errorMsg
+			};
+		}
+
+		await Asc.Editor.callMethod("EndAction", ["GroupActions"]);
+		window.Asc.plugin.sendEvent("ai_onCallToolResult", toolResult);
+	});
+
+	try {
+		if (window.AscDesktopEditor) {
+			let model = JSON.parse(window.localStorage.getItem("current-model"));
+			let provider = JSON.parse(window.localStorage.getItem("current-provider"));
+
+			if (model && provider) {
+				AI.DEFAULT_DESKTOP_MODEL = {
+					name : model.name,
+					id : AI.externalModelPrefix + model.id,
+					capabilities : model.capabilities || AI.CapabilitiesUI.Chat,
+					endpoints : [AI.Endpoints.Types.v1.Chat_Completions]
+				};
+
+				for (let i = 0, len = window.AI.InternalProviders.length; i < len; i++) {
+					let internalProvider = window.AI.InternalProviders[i];
+					if (provider.baseUrl === internalProvider.url ||
+						provider.baseUrl === (internalProvider.url + "/" + internalProvider.addon)) 
+					{
+						AI.DEFAULT_DESKTOP_MODEL.provider = window.AI.InternalProviders[i].createInstance(provider.name, 
+							internalProvider.url, provider.key, internalProvider.addon);
+						break;	
+					}
+				}
+
+				if (!AI.DEFAULT_DESKTOP_MODEL.provider) {
+					AI.DEFAULT_DESKTOP_MODEL.provider = new AI.Provider(provider.name, provider.baseUrl, provider.key);
+				}
+			};
+		}
+
+		if (AI.DEFAULT_DESKTOP_MODEL) {
+			AI.Models.push(AI.DEFAULT_DESKTOP_MODEL);
+
+			for (let key in AI.Actions) {
+				if (AI.Actions[key].capabilities === AI.CapabilitiesUI.Chat && AI.Actions[key].model === "") {
+					AI.Actions[key].model = AI.DEFAULT_DESKTOP_MODEL.id;
+				}
+			}
+		}
+	} catch (e) {
+	}
+}
+
 let initCounter = 0;
 async function initWithTranslate(counter) {
 	initCounter |= counter;
 	if (3 === initCounter) {
 		initCounter = 5;
+		await AI.loadInternalProviders();
 		await registerButtons(window);
 		Asc.Buttons.registerContextMenu();
 		Asc.Buttons.registerToolbarMenu();
@@ -491,26 +755,17 @@ async function initWithTranslate(counter) {
 			}
 		}
 
-		if (editorVersion >= 9001000 && window.isEnableDocumentGenerate) {
-
-			if (window.AscDesktopEditor && Asc.Editor.getType() === "word") {
-
-				let buttonGenerate = new Asc.ButtonToolbar(null);
-				buttonGenerate.text = "Generate";
-				buttonGenerate.icons = window.getToolBarButtonIcons("ocr");
-				
-				buttonGenerate.attachOnClick(async function(){
-					let content = await getFormGenerationPrompt();
-					window.AscDesktopEditor.generateNew("docx", "ai", content);
-				});
-
-				Asc.Buttons.updateToolbarMenu(window.buttonMainToolbar.id, window.buttonMainToolbar.name, [buttonGenerate]);
-			}			
-
-		}
-
 		if (editorVersion >= 9000004)
 			window.addSupportAgentMode(editorVersion);
+
+		initAssistants();
+		initExternalProviders();
+
+		if (window.Asc.plugin.sendEvent)
+			window.Asc.plugin.sendEvent("ai_onInit", {});
+
+		if (!window.isCheckGenerationInfo)
+			window.checkGenerationInfo();
 	}
 }
 
@@ -572,150 +827,6 @@ window.Asc.plugin.init = async function() {
 		delete window.Asc.plugin.info.aiPluginSettings;
 	}
 
-	if (this.sendEvent) {
-		this.sendEvent("ai_onInit", {});
-
-		function onLoadCustomExternalProviders(providers) {
-			for (let i = 0, len = providers.length; i < len; i++) {
-				let item = providers[i];
-				if (!item.name)
-					continue;
-
-				if (!item.content) {
-					let url = item.url || "[external]";
-					let key = item.key || "";
-					let addon = item.addon || "";
-
-					item.content = "\"use strict\";\n\
-class Provider extends AI.Provider {\n\
-	constructor() {\n\
-		super(\"" + item.name + "\", \"" + url + "\", \"" + key + "\", \"" + addon + "\");\n\
-	}\n\
-}";				}
-
-				let isError = !AI.addCustomProvider(item.content);
-				if (!isError) {
-					customProvidersWindow && customProvidersWindow.command('onSetCustomProvider', AI.getCustomProviders());
-					aiModelEditWindow && aiModelEditWindow.command('onProvidersUpdate', { providers : AI.serializeProviders() });
-				}					
-			}
-		}
-
-		this.attachEditorEvent("ai_onCustomProviders", function(providers) {
-			onLoadCustomExternalProviders(providers);
-		});
-
-		this.attachEditorEvent("ai_onCustomInit", function(obj) {
-			
-			if (obj.settingsLock !== undefined) {
-				let isSettingsRemoved = obj.settingsLock === "removed";
-				let isSettingsDisabled = obj.settingsLock === "disabled";
-
-				if (window.buttonSettings) {
-					if (window.buttonSettings.removed != isSettingsRemoved || 
-						window.buttonSettings.disabled != isSettingsDisabled) {
-						window.buttonSettings.removed = isSettingsRemoved;
-						window.buttonSettings.disabled = isSettingsDisabled;
-
-						Asc.Buttons.updateToolbarMenu(window.buttonMainToolbar.id, window.buttonMainToolbar.name, [window.buttonSettings]);
-					}
-				}
-			}
-
-			if (obj.actions) {
-				let isActionsOverride = obj.actionsOverride === true;
-
-				for (let type in obj.actions) {
-					if (!AI.Actions[type])
-						continue;
-					
-					if (!AI.Actions[type].model || isActionsOverride)
-						AI.Actions[type].model = obj.actions[type].model;
-				}
-
-				AI.ActionsSave();
-			}
-
-			let isUpdate = false;
-			if (obj.providers) {
-				let customProviders = [];
-				for (let type in obj.providers) {
-					if (!obj.providers[type].name)
-						continue;
-					customProviders.push(obj.providers[type]);
-				}
-				if (customProviders.length > 0)
-					onLoadCustomExternalProviders(customProviders);
-				isUpdate = true;
-			}
-
-			if (obj.models) {
-				for (let i = 0, len = obj.models.length; i < len; i++) {
-					let model = obj.models[i];
-					let isFound = false;
-
-					for (let j = 0, jLen = AI.Models.length; j < jLen; j++) {
-						let testModel = AI.Models[j];
-
-						if (testModel.name === model.name && 
-							testModel.provider === model.provider &&
-							testModel.id === model.id) {
-							isFound = true;
-							AI.Models[j] = model;
-							break;
-						}
-					}
-
-					if (!isFound) {
-						AI.Models.push(model);
-					}
-				}
-				isUpdate = true;
-			}
-
-			if (isUpdate)
-				AI.Storage.save();
-			
-		});
-		
-		spellchecker = new SpellChecker();
-		grammar = new GrammarChecker();
-
-		this.attachEditorEvent("onParagraphText", function(obj) {
-			if (!obj)
-				return;
-			
-			spellchecker.onChangeParagraph(obj["paragraphId"], obj["recalcId"], obj["text"], obj["annotations"]);
-			grammar.onChangeParagraph(obj["paragraphId"], obj["recalcId"], obj["text"], obj["annotations"]);
-		});
-
-		this.attachEditorEvent("onFocusAnnotation", function(obj) {
-			if (!obj)
-				return;
-		});
-
-		this.attachEditorEvent("onBlurAnnotation", function(obj) {
-			if (!obj)
-				return;
-
-			if ("spelling" === obj["name"])
-				spellchecker.onBlur();
-			else if ("grammar" === obj["name"]) 
-				grammar.onBlur();
-		});
-
-		this.attachEditorEvent("onClickAnnotation", function(obj) {
-			if (!obj)
-				return;
-
-			if ("grammar" === obj["name"])
-				grammar.onClick(obj["paragraphId"], obj["ranges"]);
-			else if ("spelling" === obj["name"])
-				spellchecker.onClick(obj["paragraphId"], obj["ranges"]);
-		});
-
-	}
-
 	await initWithTranslate(1 << 1);
 	clearChatState();
 
@@ -752,6 +863,21 @@ window.Asc.plugin.button = async function(id, windowId) {
 		}
 		return;
 	}
+	if (customAnnotationPopup && customAnnotationPopup.popup && customAnnotationPopup.popup.id === windowId)
+	{
+		switch (id) {
+			case 0:
+				await customAnnotationPopup.popup.onAccept();
+				break;
+			case 1:
+				await customAnnotationPopup.popup.onReject();
+				break;
+			default:
+				customAnnotationPopup.close();
+				break;
+		}	
+		return;
+	}
 
 	if (settingsWindow && windowId === settingsWindow.id) {
 		settingsWindow.close();
@@ -793,9 +919,12 @@ window.Asc.plugin.onThemeChanged = function(theme) {
 	customProvidersWindow && customProvidersWindow.command('onThemeChanged', theme);
 	window.chatWindow && window.chatWindow.command('onThemeChanged', theme);
 	helperWindow && helperWindow.command('onThemeChanged', theme);
+	customAssistantWindow && customAssistantWindow.command('onThemeChanged', theme);
 
 	if (textAnnotatorPopup && textAnnotatorPopup.popup)
 		textAnnotatorPopup.popup.command('onThemeChanged', theme);
+	if (customAnnotationPopup && customAnnotationPopup.popup)
+		customAnnotationPopup.popup.command('onThemeChanged', theme);
 };
 
 /**
@@ -913,6 +1042,268 @@ async function onCheckGrammarSpelling(isCurrent)
 		grammar.checkParagraphs(paraIds);
 }
 
+/**
+ * CUSTOM ASSISTANT
+ * @param {string} [assistantId] Assistant ID for editing
+ * @param {Asc.ButtonToolbar} [buttonAssistant]
+ */
+function customAssistantWindowShow(assistantId, buttonAssistant)
+{
+	if (window.customAssistantWindow) {
+		customAssistantWindowClose();
+	}
+	const actionButtonText = assistantId ? 'Save' : 'Create';
+	const description = assistantId ? 'Edit' : 'Create a new assistant';
+
+	let variation = {
+		url : "customAssistant.html",
+		description : window.Asc.plugin.tr(description),
+		isVisual : true,
+		buttons : [
+			{ text: window.Asc.plugin.tr(actionButtonText), primary: true },
+			{ text: window.Asc.plugin.tr('Cancel'), primary: false },
+		],
+		isModal : false,
+		isCanDocked: false,
+		type: "window",
+		EditorsSupport : ["word"],
+		size : [ 427, 303 ] //383
+	};
+
+	customAssistantWindow = new window.Asc.PluginWindow();
+	customAssistantWindow.attachEvent("onWindowReady", function() {
+		Asc.Editor.callMethod("ResizeWindow", [customAssistantWindow.id, [427, 303], [427, 303], [0, 0]]);
+		if (assistantId) {
+			customAssistantWindow.command('onEditAssistant', assistantId);
+		}
+	});
+	
+	customAssistantWindow.show(variation);
+
+	window.pluginsButtonsCallback = window.Asc.plugin.button;
+	window.Asc.plugin.button = async function(id, windowId, ...args) {
+		if (customAssistantWindow && windowId === customAssistantWindow.id) {
+			if (id === 0) {
+				const element = await new Promise(resolve => {
+					customAssistantWindow.attachEvent("onAddEditAssistant", resolve);
+					customAssistantWindow.command('onClickAdd');
+				});
+				if (!element) return;
+				if (buttonAssistant) {
+					buttonAssistant.text = element.name;
+					customAssistantManager.updateAssistant(element);
+				} else {
+					buttonAssistant = new Asc.ButtonToolbar(null);
+					buttonAssistant.text = element.name;
+					buttonAssistant.icons = getToolBarButtonIcons("written-plugin");
+					buttonAssistant.split = true;
+					buttonAssistant.enableToggle = true;
+					buttonAssistant.menu = [{
+						text: 'Edit',
+						id: element.id + '-edit',
+						onclick: () => customAssistantWindowShow(element.id, buttonAssistant)
+					},
+					{
+						text: 'Delete',
+						id: element.id + '-delete',
+						onclick: () => customAssistantWindowDeleteConfirm(element.id, buttonAssistant)
+					}];
+					buttonAssistant.attachOnClick(async function(){
+						customAssistantOnClickToolbarIcon(element.id, buttonAssistant);
+					});
+					customAssistantManager.createAssistant(element);
+				}
+				Asc.Buttons.updateToolbarMenu(window.buttonMainToolbar.id, window.buttonMainToolbar.name, [buttonAssistant]);
+			}
+			customAssistantWindowClose();
+		} else {
+			await window.pluginsButtonsCallback(id, windowId, ...args);
+		}
+	}
+
+	window.customAssistantWindow = customAssistantWindow;
+}
+
+function customAssistantWindowClose() {
+	if (window.customAssistantWindow) {
+		window.customAssistantWindow.close();
+		window.customAssistantWindow = null;
+	}
+	if (window.pluginsButtonsCallback) {
+		window.Asc.plugin.button = window.pluginsButtonsCallback;
+		window.pluginsButtonsCallback = null;
+	}
+}
+/**
+ * @param {string} assistantId
+ * @param {Asc.ButtonToolbar} buttonAssistant
+ */
+function customAssistantWindowDeleteConfirm(assistantId, buttonAssistant) {
+if (window.customAssistantWindow) {
+		customAssistantWindowClose();
+	}
+
+	const savedAssistants = JSON.parse(
+		localStorage.getItem("onlyoffice_ai_saved_assistants") || "[]"
+	);
+	const index = savedAssistants.findIndex((item) => item.id === assistantId);
+	const assistant = savedAssistants[index];
+
+	let variation = {
+		url : "customAssistant.html",
+		description : assistant.name + ' - ' + window.Asc.plugin.tr('Delete Assistant'),
+		isVisual : true,
+		buttons : [
+			{ text: window.Asc.plugin.tr('Yes'), primary: true },
+			{ text: window.Asc.plugin.tr('No'), primary: false },
+		],
+		isModal : true,
+		isCanDocked: false,
+		type: "window",
+		EditorsSupport : ["word"],
+		size : [ 400, 70 ]
+	};
+
+	const customAssistantWindow = new window.Asc.PluginWindow();
+	customAssistantWindow.attachEvent("onWindowReady", function() {
+		Asc.Editor.callMethod("ResizeWindow", [customAssistantWindow.id, [400, 70], [400, 70], [0, 0]]);
+		if (assistantId) {
+			let text = window.Asc.plugin.tr('Are you sure you want to delete this assistant?');
+			text += '<br>' + window.Asc.plugin.tr("This action cannot be undone.");
+			customAssistantWindow.command('onWarningAssistant', text);
+		}
+	});
+	
+	customAssistantWindow.show(variation);
+
+	window.pluginsButtonsCallback = window.Asc.plugin.button;
+	window.Asc.plugin.button = async function(id, windowId, ...args) {
+		if (customAssistantWindow && windowId === customAssistantWindow.id) {
+			if (id === 0) {
+
+				if (index !== -1) {
+					savedAssistants.splice(index, 1);
+					localStorage.setItem(
+						"onlyoffice_ai_saved_assistants",
+						JSON.stringify(savedAssistants)
+					);
+					if (buttonAssistant) {
+						buttonAssistant.removed = true;
+						Asc.Buttons.updateToolbarMenu(window.buttonMainToolbar.id, window.buttonMainToolbar.name, [buttonAssistant]);
+					}
+				}
+			}
+			customAssistantWindowClose();
+		} else {
+			await window.pluginsButtonsCallback(id, windowId, ...args);
+		}
+	}
+
+	window.customAssistantWindow = customAssistantWindow;
+}
+/**
+ * @param {string} warningText
+ * @param {localStorageCustomAssistantItem} [assistantData]
+ */
+function customAssistantWarning(warningText, assistantData) {
+	if (window.customAssistantWindow) {
+		customAssistantWindowClose();
+	}
+
+	let variation = {
+		url : "customAssistant.html",
+		description : window.Asc.plugin.tr('Warning!'),
+		isVisual : true,
+		buttons : [
+			{ text: window.Asc.plugin.tr('OK'), primary: true },
+		],
+		isModal : true,
+		isCanDocked: false,
+		type: "window",
+		EditorsSupport : ["word"],
+		size : [ 350, 76 ]
+	};
+
+	const customAssistantWindow = new window.Asc.PluginWindow();
+	customAssistantWindow.attachEvent("onWindowReady", function() {
+		Asc.Editor.callMethod("ResizeWindow", [customAssistantWindow.id, [350, 76], [350, 76], [0, 0]]);
+		customAssistantWindow.command('onWarningAssistant', warningText);
+
+	});
+	
+	customAssistantWindow.show(variation);
+
+	window.pluginsButtonsCallback = window.Asc.plugin.button;
+	window.Asc.plugin.button = async function(id, windowId, ...args) {
+		if (customAssistantWindow && windowId === customAssistantWindow.id) {
+			customAssistantWindowClose();
+		} else {
+			await window.pluginsButtonsCallback(id, windowId, ...args);
+		}
+	}
+
+	window.customAssistantWindow = customAssistantWindow;
+}
+/** 
+ * @param {string} assistantId 
+ * @param {Asc.ButtonToolbar} buttonAssistant 
+ * @returns 
+ */
+async function customAssistantOnClickToolbarIcon(assistantId, buttonAssistant)
+{
+	const isAssistantRunning = customAssistantManager.isCustomAssistantRunning(assistantId);
+	if (isAssistantRunning) {
+		customAssistantManager.stop(assistantId);
+		return;
+	}
+	let paraIds = [];
+
+	let selectedText = await Asc.Library.GetSelectedText();
+	let preloaderMessage = !!selectedText ? window.Asc.plugin.tr("Processing selection...") : window.Asc.plugin.tr("Processing document...");
+
+	Asc.scope.hasSelectedText = !!selectedText;
+	paraIds = await Asc.Editor.callCommand(function(){
+		let result = [];
+		let paragraphs;
+		if (Asc.scope.hasSelectedText) {
+			const range = Api.GetDocument().GetRangeBySelect();
+			paragraphs = range.GetAllParagraphs();
+		} else {
+			paragraphs = Api.GetDocument().GetAllParagraphs();
+		}
+		paragraphs.forEach(p => result.push(p.GetInternalId()));
+		return result;
+	});
+
+	await Asc.Editor.callMethod("StartAction", ["Block", preloaderMessage]);
+
+	const status = await customAssistantManager.run(assistantId, paraIds);
+	switch (status) {
+		case customAssistantManager.STATUSES.OK:
+			break;
+		case customAssistantManager.STATUSES.NOT_FOUND:
+			console.error("Custom assistant not found: " + assistantId);
+            customAssistantWarning(window.Asc.plugin.tr("Custom assistant is not available. Please check your configuration."));
+			//buttonAssistant.disabled = true;
+			//Asc.Buttons.updateToolbarMenu(window.buttonMainToolbar.id, window.buttonMainToolbar.name, [buttonAssistant]);
+			break;
+		case customAssistantManager.STATUSES.ERROR:
+            customAssistantWarning(
+				window.Asc.plugin.tr("Not able to perform this action. Please use prompts related to text analysis, editing, or formatting.")
+			);
+			// TODO: Add the ability to remove a button press.
+			// buttonAssistant.PRESSED = false;
+			// Asc.Buttons.updateToolbarMenu(window.buttonMainToolbar.id, window.buttonMainToolbar.name, [buttonAssistant]);
+			// customAssistantManager.stop(assistantId);
+			break;
+		case customAssistantManager.NO_AI_MODEL_SELECTED:
+			// A window with settings will appear.
+			break;		
+	}
+
+	await Asc.Editor.callMethod("EndAction", ["Block", preloaderMessage]);
+}
+
 /**
  * MODELS WINDOW
  */
